@@ -73,6 +73,13 @@ fstWriterSetPackType((void *)(long)ctx, (int)typ);
 }
 
 
+JNIEXPORT void JNICALL Java_fstAPI_fstWriterSetFileType
+  (JNIEnv *env, jobject obj, jlong ctx, jint typ)
+{
+fstWriterSetFileType((void *)(long)ctx, (int)typ);
+}
+
+
 JNIEXPORT void JNICALL Java_fstAPI_fstWriterSetRepackOnClose
   (JNIEnv *env, jobject obj, jlong ctx, jboolean enable)
 {
@@ -317,6 +324,13 @@ return((jint)fstReaderGetCurrentScopeLen((void *)(long)ctx));
 }
 
 
+JNIEXPORT jint JNICALL Java_fstAPI_fstReaderGetFileType
+  (JNIEnv *env, jobject obj, jlong ctx)
+{
+return((jint)fstReaderGetFileType((void *)(long)ctx));
+}
+
+
 JNIEXPORT jlong JNICALL Java_fstAPI_fstReaderGetTimezero
   (JNIEnv *env, jobject obj, jlong ctx)
 {
