@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 2009-2011.
+ * Copyright (c) Tony Bybell 2009-2013.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -154,6 +154,7 @@ char *s;
 unsigned char ttype;
 int sdt = FST_SDT_NONE;
 int svt = FST_SVT_NONE;
+long sxt = 0;
 
 while((h = fstReaderIterateHier(xc)))
         {
@@ -179,14 +180,15 @@ while((h = fstReaderIterateHier(xc)))
 				case FST_ST_VCD_PROGRAM:	ttype = TREE_VCD_ST_PROGRAM; break;
 
 				case FST_ST_VHDL_ARCHITECTURE:	ttype = TREE_VHDL_ST_ARCHITECTURE; break;
-				case FST_ST_VHDL_PROCEDURE:	ttype = TREE_VHDL_ST_PROCEDURE; break;
-				case FST_ST_VHDL_FUNCTION:	ttype = TREE_VHDL_ST_FUNCTION; break;
-				case FST_ST_VHDL_RECORD:	ttype = TREE_VHDL_ST_RECORD; break;
-				case FST_ST_VHDL_PROCESS:	ttype = TREE_VHDL_ST_PROCESS; break;
-				case FST_ST_VHDL_BLOCK:		ttype = TREE_VHDL_ST_BLOCK; break;
-				case FST_ST_VHDL_FOR_GENERATE:	ttype = TREE_VHDL_ST_GENFOR; break;
-				case FST_ST_VHDL_IF_GENERATE:	ttype = TREE_VHDL_ST_GENIF; break;
-				case FST_ST_VHDL_GENERATE:	ttype = TREE_VHDL_ST_GENERATE; break;
+				case FST_ST_VHDL_PROCEDURE:	ttype = TREE_VHDL_ST_PROCEDURE;    break;
+				case FST_ST_VHDL_FUNCTION:	ttype = TREE_VHDL_ST_FUNCTION;     break;
+				case FST_ST_VHDL_RECORD:	ttype = TREE_VHDL_ST_RECORD;       break;
+				case FST_ST_VHDL_PROCESS:	ttype = TREE_VHDL_ST_PROCESS;      break;
+				case FST_ST_VHDL_BLOCK:		ttype = TREE_VHDL_ST_BLOCK;        break;
+				case FST_ST_VHDL_FOR_GENERATE:	ttype = TREE_VHDL_ST_GENFOR;       break;
+				case FST_ST_VHDL_IF_GENERATE:	ttype = TREE_VHDL_ST_GENIF;        break;
+				case FST_ST_VHDL_GENERATE:	ttype = TREE_VHDL_ST_GENERATE;     break;
+				case FST_ST_VHDL_PACKAGE:	ttype = TREE_VHDL_ST_PACKAGE;      break;
 
 				default:			ttype = TREE_UNKNOWN; break;
 				}
@@ -291,6 +293,7 @@ while((h = fstReaderIterateHier(xc)))
 
 			h->u.var.svt_workspace = svt;
 			h->u.var.sdt_workspace = sdt;
+			h->u.var.sxt_workspace = sxt;
 
 			return(h);
                         break;
@@ -300,9 +303,62 @@ while((h = fstReaderIterateHier(xc)))
 				{
 				if(h->u.attr.subtype == FST_MT_SUPVAR)
 					{
+					if(h->u.attr.name)
+						{
+						JRB subvar_jrb_node;
+						char *attr_pnt;
+
+						if(GLOBALS->fst_filetype == FST_FT_VHDL)
+							{
+							char *lc_p = attr_pnt = strdup_2(h->u.attr.name);
+
+							while(*lc_p)
+								{
+								*lc_p = tolower(*lc_p); /* convert attrib name to lowercase for VHDL */
+								lc_p++;
+								}
+							}
+							else
+							{
+							attr_pnt = NULL;
+							}
+
+						/* sxt points to actual type name specified in FST file */
+						subvar_jrb_node = jrb_find_str(GLOBALS->subvar_jrb, attr_pnt ? attr_pnt : h->u.attr.name);
+						if(subvar_jrb_node)
+							{
+							sxt = subvar_jrb_node->val.ui;
+							}
+							else
+							{
+							Jval jv;
+
+							if(GLOBALS->subvar_jrb_count != WAVE_VARXT_MAX_ID)
+								{
+								sxt = jv.ui = ++GLOBALS->subvar_jrb_count;
+								/* subvar_jrb_node = */ jrb_insert_str(GLOBALS->subvar_jrb, strdup_2(attr_pnt ? attr_pnt : h->u.attr.name), jv);
+								}
+								else
+								{
+								sxt = 0;
+								if(!GLOBALS->subvar_jrb_count_locked)
+									{
+									fprintf(stderr, FST_RDLOAD"Max number (%d) of type attributes reached, please increase WAVE_VARXT_MAX_ID.\n", WAVE_VARXT_MAX_ID);
+									GLOBALS->subvar_jrb_count_locked = 1;
+									}
+								}
+							}
+
+						if(attr_pnt)
+							{
+							free_2(attr_pnt);
+							}
+						}					
+
 					svt = h->u.attr.arg >> FST_SDT_SVT_SHIFT_COUNT;
 					sdt = h->u.attr.arg & (FST_SDT_ABS_MAX-1);
 					GLOBALS->supplemental_datatypes_encountered = 1;
+					GLOBALS->supplemental_vartypes_encountered |= ((svt != FST_SVT_NONE) && (svt != FST_SVT_VHDL_SIGNAL));
 					}
 				}
 			break;
@@ -381,7 +437,16 @@ f_name_len = calloc_2(F_NAME_MODULUS+1,sizeof(int));
 f_name_max_len = calloc_2(F_NAME_MODULUS+1,sizeof(int));
 
 nnam_max = 16;
-nnam = malloc_2(nnam_max);
+nnam = malloc_2(nnam_max + 1);
+
+GLOBALS->fst_filetype = fstReaderGetFileType(GLOBALS->fst_fst_c_1);
+if(GLOBALS->fst_filetype == FST_FT_VHDL)
+	{
+	GLOBALS->is_vhdl_component_format = 1;
+	}
+
+GLOBALS->subvar_jrb = make_jrb(); /* only used for attributes such as generated in VHDL, etc. */
+
 
 GLOBALS->numfacs=fstReaderGetVarCount(GLOBALS->fst_fst_c_1);
 GLOBALS->mvlfacs_fst_c_3=(struct fac *)calloc_2(GLOBALS->numfacs,sizeof(struct fac));
@@ -587,6 +652,9 @@ for(i=0;i<GLOBALS->numfacs;i++)
 			{
 			case FST_VT_VCD_PARAMETER:
 			case FST_VT_VCD_INTEGER:
+			case FST_VT_SV_INT:
+			case FST_VT_SV_SHORTINT:
+			case FST_VT_SV_LONGINT:
 				GLOBALS->mvlfacs_fst_c_3[i].flags = VZT_RD_SYM_F_INTEGER;
 				break;	
 
@@ -748,10 +816,19 @@ for(i=0;i<GLOBALS->numfacs;i++)
 	
 			if(f->flags&VZT_RD_SYM_F_INTEGER)
 				{
-				node_block[i].msi=31;
-				node_block[i].lsi=0;
-				GLOBALS->mvlfacs_fst_c_3[i].len=32;
-				}
+				if(f->len != 0)
+					{
+					node_block[i].msi = f->len - 1;
+					node_block[i].lsi = 0;
+					GLOBALS->mvlfacs_fst_c_3[i].len = f->len;
+					}
+					else
+					{
+					node_block[i].msi = 31;
+					node_block[i].lsi = 0;
+					GLOBALS->mvlfacs_fst_c_3[i].len = 32;
+					}	
+			}
 	
 			if(GLOBALS->fast_tree_sort) 
 				{
@@ -779,6 +856,7 @@ for(i=0;i<GLOBALS->numfacs;i++)
         n->mv.mvlfac = GLOBALS->mvlfacs_fst_c_3+i;
 	GLOBALS->mvlfacs_fst_c_3[i].working_node = n;
 	n->vardir = nvd;
+	n->varxt  = h->u.var.sxt_workspace;
 	if((h->u.var.svt_workspace == FST_SVT_NONE) && (h->u.var.sdt_workspace == FST_SDT_NONE))
 		{
 		n->vartype = nvt;
@@ -846,6 +924,17 @@ free_2(f_name_len); f_name_len = NULL;
 
 if(numvars != GLOBALS->numfacs) { GLOBALS->mvlfacs_fst_rvs_alias = realloc_2(GLOBALS->mvlfacs_fst_rvs_alias, numvars * sizeof(fstHandle)); }
 
+if(GLOBALS->subvar_jrb_count) /* generate lookup table for typenames explicitly given as attributes */
+	{
+	JRB subvar_jrb_node = NULL;
+	GLOBALS->subvar_pnt = calloc_2(GLOBALS->subvar_jrb_count + 1, sizeof(char *));
+
+	jrb_traverse(subvar_jrb_node, GLOBALS->subvar_jrb)
+		{
+		GLOBALS->subvar_pnt[subvar_jrb_node->val.ui] = subvar_jrb_node->key.s;
+		}
+	}
+
 decorated_module_cleanup(); /* ...also now in gtk2_treesearch.c */
 freeze_facility_pack();
 iter_through_comp_name_table();
