@@ -315,6 +315,13 @@ NULL, /* double_curr_fst */
 NULL, /* double_fini_fst */
 0, /* nonimplicit_direction_encountered */
 0, /* supplemental_datatypes_encountered */
+0, /* supplemental_vartypes_encountered */
+0, /* is_vhdl_component_format */
+NULL, /* subvar_jrb */
+0, /* subvar_jrb_count */
+NULL, /* subvar_pnt */
+0, /* fst_filetype */
+0, /* subvar_jrb_count_locked */
 
 
 /*
@@ -597,6 +604,7 @@ NULL, /* filesel_newviewer_menu_c_1 245 */
 NULL, /* filesel_logfile_menu_c_1 246 */
 NULL, /* filesel_scriptfile_menu */
 NULL, /* filesel_writesave 247 */
+NULL, /* filesel_imagegrab */
 0, /* save_success_menu_c_1 248 */
 NULL, /* filesel_vcd_writesave 249 */
 NULL, /* filesel_lxt_writesave 250 */
@@ -1925,7 +1933,8 @@ void reload_into_new_context_2(void)
  strcpy2_into_new_context(new_globals, &new_globals->fcurr_ttranslate_c_1, &GLOBALS->fcurr_ttranslate_c_1);
  strcpy2_into_new_context(new_globals, &new_globals->fcurr_ptranslate_c_1, &GLOBALS->fcurr_ptranslate_c_1);
  strcpy2_into_new_context(new_globals, &new_globals->fcurr_translate_c_2, &GLOBALS->fcurr_translate_c_2);
- 
+
+ strcpy2_into_new_context(new_globals, &new_globals->filesel_imagegrab, &GLOBALS->filesel_imagegrab);
 
  /* renderopt.c */
 #ifdef WAVE_GTK_UNIX_PRINT
@@ -1989,6 +1998,15 @@ void reload_into_new_context_2(void)
  }
 
 
+ /* fst.c, though might be others later */
+ if(GLOBALS->subvar_jrb)
+	{
+ 	jrb_free_tree(GLOBALS->subvar_jrb);
+ 	GLOBALS->subvar_jrb = NULL;
+ 	GLOBALS->subvar_jrb_count = 0;
+	}
+
+
  /* deallocate any loader-related stuff */
  switch(GLOBALS->loaded_file_type) {
    case LXT_FILE:
