@@ -314,6 +314,13 @@ double *double_curr_fst;
 double *double_fini_fst;
 char nonimplicit_direction_encountered;
 char supplemental_datatypes_encountered;
+char supplemental_vartypes_encountered;
+char is_vhdl_component_format;
+JRB subvar_jrb;
+unsigned int subvar_jrb_count;
+char **subvar_pnt;
+unsigned char fst_filetype;
+unsigned subvar_jrb_count_locked : 1;
 
 
 /*
@@ -593,6 +600,7 @@ char *filesel_newviewer_menu_c_1; /* from menu.c 262 */
 char *filesel_logfile_menu_c_1; /* from menu.c 263 */
 char *filesel_scriptfile_menu; /* from menu.c */
 char *filesel_writesave; /* from menu.c 264 */
+char *filesel_imagegrab; /* from menu.c */
 char save_success_menu_c_1; /* from menu.c 265 */
 char *filesel_vcd_writesave; /* from menu.c 266 */
 char *filesel_lxt_writesave; /* from menu.c 267 */
