@@ -25,6 +25,7 @@
 
 WAVE_NODEVARTYPE_STR
 WAVE_NODEVARDIR_STR
+WAVE_NODEVARDATATYPE_STR
 
 enum { VIEW_DRAG_INACTIVE, TREE_TO_VIEW_DRAG_ACTIVE, SEARCH_TO_VIEW_DRAG_ACTIVE };
 
@@ -111,6 +112,14 @@ return(s);
 }
 
 
+/* truncate VHDL types to string directly after final '.' */
+char *varxt_fix(char *s)
+{
+char *pnt = strrchr(s, '.');
+return(pnt ? (pnt+1) : s);
+}
+
+
 /* Fill the store model using current SIG_ROOT and FILTER_STR.  */
 void
 fill_sig_store (void)
@@ -136,9 +145,14 @@ fill_sig_store (void)
 	int is_tname = 0;
 	int wrexm;
 	int vardt;
+	unsigned int varxt;
+	char *varxt_pnt;
 
 	if(i < 0) continue;
 
+	varxt = GLOBALS->facs[i]->n->varxt;
+	varxt_pnt = varxt ? varxt_fix(GLOBALS->subvar_pnt[varxt]) : NULL;
+
 	vartype = GLOBALS->facs[i]->n->vartype;
 	if((vartype < 0) || (vartype > ND_VARTYPE_MAX))
 		{
@@ -200,9 +214,11 @@ fill_sig_store (void)
 			gtk_list_store_set (GLOBALS->sig_store_treesearch_gtk2_c_1, &iter,
 				    NAME_COLUMN, s,
 				    TREE_COLUMN, t,
-				    TYPE_COLUMN, vartype_strings[vartype],
+				    TYPE_COLUMN, 
+						(((GLOBALS->supplemental_datatypes_encountered) && (!GLOBALS->supplemental_vartypes_encountered)) ?
+							(varxt ? varxt_pnt : vardatatype_strings[vardt]) : vartype_strings[vartype]),
 				    DIR_COLUMN, vardir_strings[vardir],
-				    DTYPE_COLUMN, vardatatype_strings[vardt],
+				    DTYPE_COLUMN, varxt ? varxt_pnt : vardatatype_strings[vardt],
 				    -1);
 
 			if(s != t->name)
@@ -220,9 +236,11 @@ fill_sig_store (void)
 			gtk_list_store_set (GLOBALS->sig_store_treesearch_gtk2_c_1, &iter,
 				    NAME_COLUMN, s,
 				    TREE_COLUMN, t,
-				    TYPE_COLUMN, vartype_strings[vartype],
+				    TYPE_COLUMN, 
+						(((GLOBALS->supplemental_datatypes_encountered) && (!GLOBALS->supplemental_vartypes_encountered)) ?
+							(varxt ? varxt_pnt : vardatatype_strings[vardt]) : vartype_strings[vartype]),
 				    DIR_COLUMN, vardir_strings[vardir],
-				    DTYPE_COLUMN, vardatatype_strings[vardt],
+				    DTYPE_COLUMN, varxt ? varxt_pnt : vardatatype_strings[vardt],
 				    -1);
 			}
       		}
@@ -1433,12 +1451,12 @@ do_tooltips:
 		case VCD_FILE:
 		case VCD_RECODER_FILE:
 		case DUMPLESS_FILE:
-					column = gtk_tree_view_column_new_with_attributes (GLOBALS->supplemental_datatypes_encountered ? "VType" : "Type",
+					column = gtk_tree_view_column_new_with_attributes (((GLOBALS->supplemental_datatypes_encountered) && (GLOBALS->supplemental_vartypes_encountered)) ? "VType" : "Type",
 							   renderer,
 							   "text", TYPE_COLUMN,
 							   NULL);
 					gtk_tree_view_append_column (GTK_TREE_VIEW (sig_view), column);
-					if(GLOBALS->supplemental_datatypes_encountered)
+					if((GLOBALS->supplemental_datatypes_encountered) && (GLOBALS->supplemental_vartypes_encountered))
 						{
 						column = gtk_tree_view_column_new_with_attributes ("DType",
 							   renderer,
@@ -1712,12 +1730,12 @@ GtkWidget* treeboxframe(char *title, GtkSignalFunc func)
 		case VCD_FILE:
 		case VCD_RECODER_FILE:
 		case DUMPLESS_FILE:
-					column = gtk_tree_view_column_new_with_attributes (GLOBALS->supplemental_datatypes_encountered ? "VType" : "Type",
+					column = gtk_tree_view_column_new_with_attributes (((GLOBALS->supplemental_datatypes_encountered) && (GLOBALS->supplemental_vartypes_encountered)) ? "VType" : "Type",
 							   renderer,
 							   "text", TYPE_COLUMN,
 							   NULL);
 					gtk_tree_view_append_column (GTK_TREE_VIEW (sig_view), column);
-					if(GLOBALS->supplemental_datatypes_encountered)
+					if((GLOBALS->supplemental_datatypes_encountered) && (GLOBALS->supplemental_vartypes_encountered))
 						{
 						column = gtk_tree_view_column_new_with_attributes ("DType",
 							   renderer,
